"""Consts used by Tradfri."""
from homeassistant.const import CONF_HOST  # noqa pylint: disable=unused-import

CONF_IMPORT_GROUPS = "import_groups"
CONF_IDENTITY = "identity"
CONF_KEY = "key"
CONF_GATEWAY_ID = "gateway_id"
