"""Constant for AWS component."""
DOMAIN = "aws"

DATA_CONFIG = "aws_config"
DATA_HASS_CONFIG = "aws_hass_config"
DATA_SESSIONS = "aws_sessions"

CONF_ACCESS_KEY_ID = "aws_access_key_id"
CONF_CONTEXT = "context"
CONF_CREDENTIAL_NAME = "credential_name"
CONF_CREDENTIALS = "credentials"
CONF_NOTIFY = "notify"
CONF_PROFILE_NAME = "profile_name"
CONF_REGION = "region_name"
CONF_SECRET_ACCESS_KEY = "aws_secret_access_key"
CONF_SERVICE = "service"
CONF_VALIDATE = "validate"
